@@ -32,8 +32,20 @@ class A(TypedDict):
     val: str
 
 
+class B(TypedDict, total=False):
+    val: str
+
 class TestTypeddictLoad(unittest.TestCase):
 
+    def test_totality(self):
+        with self.assertRaises(ValueError):
+            load({}, A)
+        assert load({}, B) == {}
+        assert load({'val': 'a'}, B) == {'val': 'a'}
+        assert load({'vel': 'q'}, B) == {}
+        with self.assertRaises(ValueError):
+            load({'vel': 'q'}, B, failonextra=True)
+
     def test_loadperson(self):
         o = {'name': 'pino', 'age': 1.1}
         assert load(o, Person) == o
@@ -50,3 +62,4 @@ class TestTypeddictLoad(unittest.TestCase):
     def test_is_typeddict(self):
         assert typechecks.is_typeddict(A)
         assert typechecks.is_typeddict(Person)
+        assert typechecks.is_typeddict(B)
