@@ -424,7 +424,7 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
         return tuple(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, (v, t) in enumerate(zip(value, args)))
 
 
-def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any]) -> Dict[str, Any]:
+def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any], failonextra: bool) -> Dict[str, Any]:
     """
     Mangling names of a dictionary.
 
@@ -441,14 +441,17 @@ def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any]) -> Dict[str,
 
     for k, v in value.items():
         if k in skip and k not in namesmap:
-            continue
+            if failonextra:
+                raise ValueError('Extra field: %s' % k)
+            else:
+                continue
         if k in namesmap:
             k = namesmap[k]
         r[k] = v
     return r
 
 
-def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
+def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Any:
     """
     This loads a Dict[str, Any] into a NamedTuple.
     """
@@ -460,7 +463,10 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
         #dataclass
         import dataclasses
         fields = set(type_.__dataclass_fields__.keys())
-        optional_fields = {k for k,v in type_.__dataclass_fields__.items() if not (isinstance(getattr(v, 'default', dataclasses._MISSING_TYPE()), dataclasses._MISSING_TYPE) and isinstance(getattr(v, 'default_factory', dataclasses._MISSING_TYPE()), dataclasses._MISSING_TYPE))}
+        optional_fields = {k for k,v in type_.__dataclass_fields__.items() if
+                           v.init == False or
+                           not isinstance(v.default, dataclasses._MISSING_TYPE) or
+                           not isinstance(v.default_factory, dataclasses._MISSING_TYPE)}
         type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
 
         #Name mangling
@@ -473,10 +479,18 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
                 if name:
                     transforms[name] = pyname
 
-        value = _mangle_names(transforms, value)
+        try:
+            value = _mangle_names(transforms, value, l.failonextra)
+        except ValueError as e:
+            raise TypedloadValueError(str(e), value=value, type_=type_)
 
+    if getattr(type_, '__total__', True) == False:
+        # For TypedDict, total=False means that fields can be safely skipped
+        # So we set the necessary_fields to empty.
+        necessary_fields = set()
+    else:
+        necessary_fields = fields.difference(optional_fields)
 
-    necessary_fields = fields.difference(optional_fields)
     try:
         vfields = set(value.keys())
     except AttributeError as e:
@@ -510,7 +524,12 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
             type_hints[k],
             annotation=Annotation(AnnotationType.FIELD, k),
         )
-    return type_(**params)
+    try:
+        return type_(**params)
+    except TypeError as e:
+        raise TypedloadTypeError(e)
+    except ValueError as e:
+        raise TypedloadValueError(e)
 
 
 def _unionload(l: Loader, value, type_) -> Any:
@@ -626,8 +645,10 @@ def _attrload(l, value, type_):
         if l.mangle_key in attribute.metadata:
             namesmap[attribute.metadata[l.mangle_key]] = attribute.name
 
-    value = _mangle_names(namesmap, value)
-
+    try:
+        value = _mangle_names(namesmap, value, l.failonextra)
+    except ValueError as e:
+        raise TypedloadValueError(str(e), value=value, type_=type_)
 
     t = _FakeNamedTuple((
         tuple(names),
